import random
import string
from datetime import datetime


def random_table_name(prefix):
    suffix = "".join(random.choices(string.ascii_letters + string.digits, k=6))
    current_datetime = datetime.now().strftime("%Y%m%d-%H%M%S")
    return f"{prefix}{current_datetime}-{suffix}"



from google.cloud import bigquery
from datetime import date, timedelta, datetime, timezone
from google.cloud.bigquery import Table
from .export_table import ExportTask


class CreateTableResult:
    def __init__(self, client, table_id):
        self.client = client
        self.table_id = table_id

    def expires_in(self, days):
        table = self.client.get_table(Table.from_string(self.table_id))
        expiration = datetime.now(timezone.utc) + timedelta(days=days)
        table.expires = expiration
        table = self.client.update_table(table, ["expires"])
        return self

    def export(self):
        return ExportTask(table_id=self.table_id)


def create_table(
    table_id, query, query_parameters=[], overwrite=False, project=None, client=None
):
    c = client
    if c is None:
        c = bigquery.Client(project=project)

    write_disposition = bigquery.WriteDisposition.WRITE_EMPTY
    if overwrite:
        write_disposition = bigquery.WriteDisposition.WRITE_TRUNCATE
    job_config = bigquery.QueryJobConfig(
        destination=table_id,
        write_disposition=write_disposition,
        query_parameters=query_parameters,
    )
    query_job = c.query(query, job_config=job_config)
    query_job.result()

    return CreateTableResult(c, table_id)



from .create_table import create_table, CreateTableResult
from .export_table import export_table, ExportTask, CSVExport
from .select import select, select_with_template, SelectTask
from .naming import random_table_name



import unittest
import bqhus
from dotenv import load_dotenv

load_dotenv()


class TestSelect(unittest.TestCase):
    def test_select_with_no_params(self):
        query = "SELECT 1"
        task = bqhus.select(query)
        self.assertEqual(query, task.query)

    def test_select(self):
        query = "SELECT word FROM `bigquery-public-data.samples.shakespeare` order by rand() limit 5"
        words = bqhus.select(query).as_dicts()
        self.assertEqual(len(words), 5)

    def test_select_with_template(self):
        words = (
            bqhus.select_with_template("./tests/templates", "shakespeare.j2")
            .int64_param("max_count", 3)
            .as_dicts()
        )
        self.assertEqual(len(words), 3)

    def test_select_to_dataframe(self):
        query = "SELECT word FROM `bigquery-public-data.samples.shakespeare` order by rand() limit 5"
        df = bqhus.select(query).to_dataframe()
        self.assertEqual(len(df.index), 5)

    def test_select_int64_param(self):
        query = "SELECT word FROM `bigquery-public-data.samples.shakespeare` order by rand() limit @max_count"
        select_task = bqhus.select(query)
        self.assertEqual(len(select_task.query_parameters), 0)
        select_task.int64_param("max_count", 5)
        self.assertEqual(len(select_task.query_parameters), 1)


if __name__ == "__main__":
    unittest.main()



from .select import select


class Bqhus:
    def __init__(self, bq_client):
        self.bq_client = bq_client

    def select(self, query, query_parameters):
        return select(
            client=self.bq_client, query=query, query_parameters=query_parameters
        )



import unittest
from bqhus import random_table_name


class TestNaming(unittest.TestCase):
    def test_uniqueness(self):
        names = [random_table_name("uniq-test") for x in range(0, 100)]
        self.assertEqual(len(names), len(set(names)))


if __name__ == "__main__":
    unittest.main()



from google.cloud import bigquery
from jinja2 import Template, Environment, FileSystemLoader
from .create_table import create_table
import google.cloud.bigquery


class SelectTask:
    def __init__(self, client, query):
        self.client = client
        self.query = query
        self.query_parameters = []

    def params(self, params):
        self.query_parameters = params
        return self

    def int64_param(self, key: str, value: int):
        if self.query_parameters is None:
            self.query_parameters = []

        self.query_parameters.append(
            bigquery.ScalarQueryParameter(key, "INT64", value),
        )
        return self

    def str_param(self, key: str, value: str):
        if self.query_parameters is None:
            self.query_parameters = []

        self.query_parameters.append(
            bigquery.ScalarQueryParameter(key, "STRING", value),
        )
        return self

    def bq_client(self):
        return self.client if self.client is not None else bigquery.Client()

    def job(self):
        job_config = bigquery.QueryJobConfig(query_parameters=self.query_parameters)
        return self.bq_client().query(self.query, job_config=job_config)

    def as_dicts(self):
        query_job = self.job()
        return [dict(row) for row in query_job]

    def first_as_dict(self):
        return self.as_dicts()[0]

    def to_table(self, table_id, overwrite=False):
        return create_table(
            table_id,
            self.query,
            self.query_parameters,
            overwrite=overwrite,
            client=self.client,
        )

    def to_dataframe(self):
        return self.job().to_dataframe()


def select(
    query: str,
    query_parameters=[],
    client: google.cloud.bigquery.Client = None,
):
    return SelectTask(client=client, query=query).params(query_parameters)


def select_with_template(
    template_dir: str, template_name: str, template_parameters={}, client=None
):
    env = Environment(loader=FileSystemLoader(template_dir, encoding="utf8"))
    tmpl = env.get_template(template_name)
    query = tmpl.render(template_parameters)

    return select(query=query, query_parameters=[], client=client)



from google.cloud import bigquery
from google.cloud.bigquery import Table


class CSVExport:
    def __init__(self, task, field_delimiter, gzip, print_header):
        self.task = task
        self.field_delimiter = field_delimiter
        self.gzip = gzip
        self.print_header = print_header

    def to_gcs(self, bucket, object_name):
        destination_uri = f"gs://{bucket}/{object_name}"

        c = self.task.get_bq_client()

        job_config = bigquery.job.ExtractJobConfig()
        if self.gzip:
            job_config.compression = bigquery.Compression.GZIP
        job_config.field_delimiter = self.field_delimiter
        job_config.print_header = self.print_header

        extract_job = c.extract_table(
            Table.from_string(self.task.table_id),
            destination_uri,
            location=self.task.location,
            job_config=job_config,
        )

        extract_job.result()

        return destination_uri


class ExportTask:
    def __init__(self, table_id, location=None, project=None, client=None):
        self.table_id = table_id
        self.location = location
        self.project = project
        self.bq_client = client

    def get_bq_client(self):
        if self.bq_client is not None:
            return self.bq_client
        return bigquery.Client(project=self.project)

    def as_csv(self, field_delimiter=",", gzip=False, print_header=True):
        return CSVExport(
            task=self,
            field_delimiter=field_delimiter,
            gzip=gzip,
            print_header=print_header,
        )


def export_table(table_id, location=None, project=None, client=None):
    return ExportTask(
        table_id=table_id, location=location, project=project, client=client
    )



# Author: yokoe <kreuz45@kreuz45.com>
# Copyright (c) 2023- yokoe
# Licence: MIT

from setuptools import setup

DESCRIPTION = "bqhus: BigQuery helper utils."
NAME = "bqhus"
AUTHOR = "yokoe"
AUTHOR_EMAIL = "kreuz45@kreuz45.com"
URL = "https://github.com/yokoe/bqhus"
LICENSE = "MIT"
DOWNLOAD_URL = URL
VERSION = "0.1.9"
PYTHON_REQUIRES = ">=3.9"
INSTALL_REQUIRES = [
    "google-cloud-bigquery>=3.4.0",
    "Jinja2>=3.0.0",
    "pandas>=1.4.0",
    "db-dtypes>=1.0.5",
]
PACKAGES = ["bqhus"]
PACKAGE_DIR = {"": "src"}
KEYWORDS = "bigquery"
CLASSIFIERS = [
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 3.6",
]
with open("README.md", "r", encoding="utf-8") as fp:
    readme = fp.read()
LONG_DESCRIPTION = readme
LONG_DESCRIPTION_CONTENT_TYPE = "text/markdown"

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESCRIPTION_CONTENT_TYPE,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    maintainer=AUTHOR,
    maintainer_email=AUTHOR_EMAIL,
    url=URL,
    download_url=URL,
    packages=PACKAGES,
    package_dir=PACKAGE_DIR,
    classifiers=CLASSIFIERS,
    license=LICENSE,
    keywords=KEYWORDS,
    install_requires=INSTALL_REQUIRES,
)
